% For use with Matlab
% Copyright (C) 2019 -- Jérôme Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araújo <rui@isr.uc.pt>
% Jérôme Mendes, Francisco Souza, and Rui Araújo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jérôme Mendes, Francisco Souza and Rui Araújo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [psi] = Psi(data_input, x)
%% Description
% This function obtains the variable \Psi (equation (11) )

%% Function inputs
% x - Struct with all fuzzy control parameters
% data_input -  values of the input variable

%% Function output
% psi - variable \psi - equation (11)

n_var= size(x, 2); % number of inputs variables
k = 1;

for j=1:n_var % for all input variables

    n_rules = size( x(j).Rules,1 ); % number of fuzzy rules on input variable j

    for i = 1:n_rules % for all fuzzy rules
        
        % MF parameters of the fuzzy rule i of input variable j
        parameters(1,1) = x(j).Rules(i,1); % lower limit 
        parameters(1,2) = x(j).Rules(i,2); % center
        parameters(1,3) = x(j).Rules(i,3); % upper limit

        mu_varialble(k) = trifp(data_input(j), parameters); % fuzzy degree on the MF of the fuzzy rule i of input variable j
        k = k + 1;
    end
end

psi = mu_varialble / (sum(mu_varialble) + 0.000000001); % \psi - equation 11 (0.000000001 is just a protection....)

end